require 'json'
require 'typhoeus'
require 'ostruct'

module GW2API
	class Endpoint
		def initialize
			# The URL of the API endpoint
			@url = GW2API::BASE_URL
			# Does this endpoint support ?id= and ?ids=?
			@bulk_expandable = false
			# Does this endpoint support ?ids=all?
			@bulk_expandable_all = false
			# Does this endpoint support ?page= and ?page_size=?
			@paginated = false
			# Does this endpoint support ?language=?
			@localized = false
			# Does this endpoint require an API key?
			@authenticated = false
			# Does this endpoint optionally allow an API key?
			@authenticated_optional = false
		end

		def ids
			return nil unless @bulk_expandable
			api_call "#{@url}?ids=all"
		end

		def get(id = nil)
			return nil if id.nil? && @bulk_expandable
			if id.kind_of?(Array)
				api_call("#{@url}?ids=#{id.join(',')}")
			else
				api_call("#{@url}/#{id}")
			end
		end

		def all
			return nil unless @bulk_expandable_all
			api_call("#{@url}?ids=all")
		end

		def api_call(url)
			request = Typhoeus::Request.new(url)
			request.on_complete do |resp|
				if resp.success?
					return JSON.parse(resp.body, object_class: OpenStruct)
				else
					return nil
				end
			end
			request.run
		end
	end
end
